/*
 * CHH  98/02/06   H_TOP02
 */

/* these two files are indactes information to the hosts */
#include "bi_codes.h"
#include "macros.ddl"

 #define ALL_UNITS   \
		UNITS_CODE (32), \
		UNITS_CODE (33), \
		UNITS_CODE (36), \
		UNITS_CODE (37)


MANUFACTURER KAMSTRUP, DEVICE_TYPE _FLEXTOP_HRT, DEVICE_REVISION 2, DD_REVISION 1


IMPORT STANDARD _TABLES, DEVICE_REVISION 8, DD_REVISION 1
{
	EVERYTHING;
}

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 1
{
	EVERYTHING;

	REDEFINITIONS
	{
				VARIABLE device_type
				{
						REDEFINE TYPE ENUMERATED
						{
								{	239,  "FlexTop HRT" }
						}
				}
				VARIABLE manufacturer_id
				{
						REDEFINE TYPE ENUMERATED
						{
								{ 96,  "Kamstrup" }
						}
				}
				VARIABLE private_label_distributor
				{
						REDEFINE TYPE ENUMERATED
						{
								{ 96,  "Kamstrup" }
						}
				}


				COMMAND read_dynamic_variables_and_pv_current
				{
						REDEFINE TRANSACTION
						{
								REQUEST
								{}
								REPLY
								{
										response_code, device_status,
										PV.ANALOG_VALUE, PV.DIGITAL_UNITS, PV.DIGITAL_VALUE
								}
						}
				}
	}
}

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 2
{

	COMMAND write_pv_damping_value; 				 			/*command  34*/
	COMMAND write_pv_range_values;					 			/*Command  35*/
	COMMAND set_pv_upper_range_value;				 			/*command  36*/
	COMMAND set_pv_lower_range_value;				 			/*command  37*/
	COMMAND write_pv_transfer_function;			 			/*command  47*/

	/* Loop test */
	VARIABLE analog_output_fixed1;
	COMMAND enter_exit_fixed_pv_current_mode; 		/*command 40*/
	METHOD loop_test;
	METHOD leave_fixed_current_mode;
	/* UNIT */
	COMMAND write_pv_units;                       /* command 44 */
	/*DA TRIM*/
	COMMAND trim_pv_current_dac_zero;         		/*command  45*/
	COMMAND trim_pv_current_dac_gain;         		/*command  46*/
	METHOD dac_trim;

	/* Status kode */
	VARIABLE xmtr_specific_status_0;
	VARIABLE xmtr_specific_status_1;
  COMMAND read_additional_transmitter_status;   /*command  48*/

	/* Sensor trim */
	METHOD return_to_normal;

	REDEFINITIONS
	{

			VARIABLE xmtr_specific_status_0
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{ 0x01,"Sensor Error",  HARDWARE }
			}
		}


		VARIABLE xmtr_specific_status_1
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{ 0x01,"",              HARDWARE }
			}
		}

		COMMAND read_additional_transmitter_status
		{
			 REDEFINE TRANSACTION
			 {
					REQUEST
					{}
					REPLY
					{
						 response_code,
						 device_status,
						 xmtr_specific_status_0,
						 xmtr_specific_status_1
					}
			 }
		}
	}
}


/*
 * Dynamic Variables Configuration
 */


VARIABLE sensor_value
{
	HELP [digital_value_help];
	CLASS CORRECTION & DYNAMIC;
	LABEL
		SELECT (sensor_type)
		{
			CASE 1:
					"Volt";
			CASE 2:
					"Volt.";
			CASE 3:
					"Volt.";
			CASE 4:
					"Volt.";
			CASE 5:
					"Res.";
			CASE 6:
					"Res.";
			CASE 7:
					"Res.";
			CASE 8:
					"Res.";
			CASE 9:
					"Res.";
			CASE 10:
					"Res.";
			CASE 11:
					"Res.";
			CASE 12:
					"Res.";
			DEFAULT:
					"Temp.";
		}
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 1:
					".2f";
				CASE 2:
					".1f";
				CASE 3:
					".2f";
				CASE 4:
					".1f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".2f";
				CASE 14:
					".2f";
				CASE 15:
					".2f";
				CASE 28:
					".2f";
				CASE 29:
					".2f";
				DEFAULT:
					".1f";
			}
	}
}

VARIABLE sensor_units
{
	HELP [digital_units_help];
	CLASS CORRECTION;
	LABEL "Unit" "|049Einheit";
	HANDLING READ&WRITE;
	TYPE ENUMERATED
	{
		ALL_UNITS
	}

	VALIDITY
	IF (sensor_type > 12){TRUE;}
	ELSE {FALSE;}
}

VARIABLE sensor_upper_limit
{
	HELP [upper_sensor_limit_help];
	CLASS CORRECTION;
	LABEL "USL" "|049OSL";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 2:
					".0f";
				CASE 4:
					".0f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".1f";
				CASE 14:
					".1f";
				CASE 15:
					".1f";
DEFAULT:
					".1f";
			}
	}
}

VARIABLE sensor_lower_limit
{
	HELP [lower_sensor_limit_help];
	CLASS CORRECTION;
	LABEL "LSL" "|049USL";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 2:
					".0f";
				CASE 4:
					".0f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".1f";
				CASE 14:
					".1f";
				CASE 15:
					".1f";

				DEFAULT:
					".1f";
			}
	}
}

VARIABLE sensor_minimum_span
{
	HELP [minimum_span_help];
	CLASS CORRECTION;
	LABEL "MinSpan" "|049MinSpan";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 2:
					".0f";
				CASE 4:
					".0f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".1f";
				CASE 14:
					".1f";
				CASE 15:
					".1f";
				DEFAULT:
					".1f";
			}
	}
}

VARIABLE sensor_damping_value
{
	HELP [seconds_damping_value_help];
	LABEL [damping_value];
	CLASS CORRECTION;
	HANDLING READ&WRITE;
	CONSTANT_UNIT "Sec." "|049Sek.";
	TYPE FLOAT
	{
		DISPLAY_FORMAT ".0f";
		EDIT_FORMAT "2.f";
		MIN_VALUE 0.0;
		MAX_VALUE 15.0;
	}
}

VARIABLE analog_value
{
	HELP [analog_value_help];
	CLASS ANALOG_OUTPUT & DYNAMIC;
	LABEL "I Out" "|049I ausg.";
	HANDLING READ&WRITE;
	CONSTANT_UNIT [mA];
	TYPE FLOAT
	{
		DISPLAY_FORMAT ".3f";
		EDIT_FORMAT "6f";
		MIN_VALUE 3.5;
		MAX_VALUE 23.0;
	}
}

VARIABLE analog_alarm_code
{
	HELP [alarm_code_help];
	CLASS ANALOG_OUTPUT;
	LABEL [alarm_select];
	HANDLING READ;
	TYPE ENUMERATED
	{
		ALARM_SELECTION_CODE(0),
		ALARM_SELECTION_CODE(1)
	}
}

VARIABLE input_percent_range
{
	HELP [percent_of_range_help];
	CLASS INPUT & DYNAMIC;
	LABEL "% range" "|049% Mespan.";
	CONSTANT_UNIT [Percent];
	HANDLING READ;
	TYPE FLOAT
	{
		DISPLAY_FORMAT ".1f";
	}
}


VARIABLE input_units
{
	HELP [range_units_help];
	CLASS INPUT;
	LABEL "Range Unit" "|049MeEinheit";
	HANDLING READ&WRITE;
	TYPE ENUMERATED
	{
		ALL_UNITS
	}
}


VARIABLE input_upper_value
{
	HELP [upper_range_value_help];
	LABEL "URV" "|049ME" ;
	CLASS INPUT;
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 2:
					".0f";
				CASE 4:
					".0f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".1f";
				CASE 14:
					".1f";
				CASE 15:
					".1f";
				DEFAULT:
					".1f";
			}
		EDIT_FORMAT "7.f";

		/*
			MIN_VALUE sennsor_lower_limit;
			MAX_VALUE sensor_upper_limit+0.001;
		*/
	}
}


VARIABLE input_lower_value
{
	HELP [lower_range_value_help];
	LABEL "LRV" "|049MA";
	CLASS INPUT;
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 2:
					".0f";
				CASE 4:
					".0f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".1f";
				CASE 14:
					".1f";
				CASE 15:
					".1f";
				DEFAULT:
					".1f";
			}
		EDIT_FORMAT "7.f";
		/*
		MIN_VALUE sensor_lower_limit;
		MAX_VALUE sensor_upper_limit+0.001;
		*/
	}
}



VARIABLE input_transfer_function
{
	HELP [transfer_function_help];
	CLASS INPUT;
	LABEL "";
	HANDLING READ;
	TYPE ENUMERATED
	{
		TRANSFER_FUNCTION_CODE (0),		/* Linear */
		TRANSFER_FUNCTION_CODE (1),		/* Square Root */
		TRANSFER_FUNCTION_CODE (2),
		TRANSFER_FUNCTION_CODE (3),
		TRANSFER_FUNCTION_CODE (4)
	}
}
VARIABLE sensor_serial_number
{
	HELP [sensor_serial_number_help];
	CLASS DEVICE;
	LABEL [pv_sensor_serial_number];
	HANDLING READ&WRITE;
	TYPE UNSIGNED_INTEGER (3)
	{
		DISPLAY_FORMAT  "7d";
	}
}


/*************************************************************************/
/*                 Transmitter specific variables/commands               */
/*************************************************************************/


VARIABLE sensor_type
	 {
	 LABEL "Sensor";
	 HELP "input sensor type mV,Ohm,RTD,T/C "
	      "|049Eingangssensortyp mV, Ohm, RTD, T/C";
	 CLASS INPUT;
	 HANDLING READ&WRITE;

	 TYPE ENUMERATED
			{
				{	1, "mV Lo"},
				{	2, "mV Hi"},
				{	3, "mV Lo lin"},
				{	4, "mV Hi lin"},
				{	5, "Ohm Lo"},
				{	6, "Ohm Hi"},
				{	7, "Pot Lo"},
				{	8, "Pot Hi"},
				{	9, "Ohm Lo lin"},
				{10, "Ohm Hi lin"},
				{11, "Pot Lo lin"},
				{12, "Pot Hi lin"},
				{13, "Pt a=3850"},
				{14, "NI a=1618"},
				{15, "CU a=1428"},
				{16, "T/C B"},
				{17, "T/C E"},
				{18, "T/C J"},
				{19, "T/C K"},
				{20, "T/C L"},
				{21, "T/C N"},
				{22, "T/C R"},
				{23, "T/C S"},
				{24, "T/C T"},
				{25, "T/C U"},
				{26, "T/C W3"},
				{27, "T/C W5"},
				{28, "Pt a=3902"},
				{29, "Pt a=3916"}
			}
	 }

VARIABLE sensor_type_var1 /* variable for input calculation */
	 {
	 LABEL "Local variable. Do not change"
				 "|049Lokale Variable. Darf nicht gendert werden";
	 HELP "";
	 CLASS INPUT;
	 HANDLING READ&WRITE;

	 TYPE UNSIGNED_INTEGER (1);
	 }

VARIABLE sensor_type_var2   /* variable for input calculation */
	 {
	 LABEL "Local variable. Do not change"
				 "|049Lokale Variable. Darf nicht gendert werden";
	 HELP "";
	 CLASS INPUT;
	 HANDLING READ&WRITE;

	 TYPE UNSIGNED_INTEGER (1);
	 }


COMMAND read_temperatur_type_data
	 {
	 NUMBER 130;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 sensor_type,sensor_type_var1,sensor_type_var2
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }

VARIABLE dummy_sensor_type /* variable for updating variables */
	 {
	 LABEL "Local variable. Do not change"
				 "|049Lokale Variable. Darf nicht gendert werden";
	 HELP "";
	 CLASS INPUT;
	 HANDLING READ&WRITE;



	 TYPE UNSIGNED_INTEGER (1);
	 }

VARIABLE dummy_unit /* unit read is command 44 (prod.rev 2)*/
	 {
	 LABEL "Local variable. Do not change"
				 "|049Lokale Variable. Darf nicht gendert werden";

	 HELP "";
	 CLASS INPUT;
	 HANDLING READ&WRITE;



	 TYPE UNSIGNED_INTEGER (1);
	 }



COMMAND write_temperatur_type_data
	 {
	 NUMBER 131;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 dummy_sensor_type,sensor_type_var1,sensor_type_var2,
				 sensor_lower_limit,sensor_upper_limit,sensor_minimum_span,min_value_messuret,
				 max_value_messuret,dummy_unit
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 dummy_sensor_type,sensor_type_var1,sensor_type_var2,
				 sensor_lower_limit,sensor_upper_limit,sensor_minimum_span,min_value_messuret,
				 max_value_messuret,dummy_unit
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			2,	DATA_ENTRY_ERROR,	[invalid_selection];
			5,	MISC_ERROR,			[too_few_data_bytes_recieved];
			}
	 }

VARIABLE wire_numbers
	 {
	 LABEL
	 IF ((sensor_type > 15) && (sensor_type < 28)) {"CJC" "|049CJC";}
	 ELSE {"Wire No." "|049Leiteranzahl";}
	 HELP
	 IF ((sensor_type > 15) && (sensor_type < 28)) {
"CJC = 0 is internal compensation        CJC = 1 is fixed compensation         CJC 2-3 = 2-3 wires extern compensation"
"|049CJC = 0 ist interne Kompensation   CJC = 1 ist feste Kompensation     CJC 2-3 = 2-3 Leiter externe Kompensation" ;}
	 ELSE  {"Numbers of wires for Ohm/RTD measurement (2-4)"
                         " |049Leitungsanzahl fur Ohm/RTD Messungen (2-4)";}
	 CLASS INPUT;
	 HANDLING READ&WRITE;
	 TYPE UNSIGNED_INTEGER (1)
		 {
				MAX_VALUE
				IF ((sensor_type < 16) || (sensor_type > 27))
				{4;}
				ELSE
				{3;}

				MIN_VALUE
				IF ((sensor_type < 16) || (sensor_type > 27))
				{2;}
				ELSE
				{0;}
			}
	 VALIDITY
			IF (sensor_type > 4 ) {TRUE;}
			ELSE {FALSE;}
	 }


VARIABLE rtd_value
	 {
	 LABEL "RTD value"
				 "|049RTD Wert";
	 HELP "RTD value 25 .. 1000 Ohm"
				"|049RTD Wert 25 .. 1000 Ohm";
	 CLASS INPUT;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
			{
			DISPLAY_FORMAT "4.f";
			EDIT_FORMAT "4.f";
			MIN_VALUE 25.0;
			MAX_VALUE 1000.0;
			}
	 CONSTANT_UNIT "Ohm";
	 VALIDITY
			IF (((sensor_type > 11) && (sensor_type < 16)) || (sensor_type > 27)) {TRUE;}
			ELSE {FALSE;}
	 }


VARIABLE cable_resistance
	 {
	 LABEL "Cable R."
				 "|049Kabel R";
	 HELP " Two wire cable compensation 0.0 .. 60.0 Ohm"
				"|049Zweileiter Kabelkompensaton 0,0 .. 60,0 Ohm";
	 CLASS INPUT;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
			{
			DISPLAY_FORMAT "2.2f";
			EDIT_FORMAT "5.f";
			MIN_VALUE 0.0;
			MAX_VALUE 60.0;
			}
	 CONSTANT_UNIT "Ohm";
	 VALIDITY
			IF ((sensor_type > 4)&&(wire_numbers == 2)) {TRUE;}
			ELSE {FALSE;}
	 }



COMMAND read_temperatur_device_data
	 {
	 NUMBER 132;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 wire_numbers,rtd_value,cable_resistance,dummy_sensor_type
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }


COMMAND write_temperatur_device_data
	 {
	 NUMBER 133;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 wire_numbers,rtd_value,cable_resistance,sensor_type
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 wire_numbers,rtd_value,cable_resistance,sensor_type
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			}
	 }


VARIABLE under_min_sensor_value
	 {
	 LABEL "Under Lim"
				 "|049Unter Gr.";
	 HELP "Input under LSL indication 3.50 .. 4.00 mA"
				"|049Eingang unter USL Angabe 3,50 .. 4,00 mA";
	 CLASS ANALOG_OUTPUT;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
			{
			DISPLAY_FORMAT "2.2f";
			EDIT_FORMAT "5f";
			MIN_VALUE 3.50;
			MAX_VALUE 4.00;
			}
	 CONSTANT_UNIT "mA";
	 }


VARIABLE over_max_sensor_value
	 {
	 LABEL "Over Lim"
				 "|049ber Gr.";
	 HELP "Input over USL indication 20.00 .. 23.00 mA"
				"|049Eingang ber OSL Angabe 20,00 .. 23,00 mA";
	 CLASS ANALOG_OUTPUT;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
			{
			DISPLAY_FORMAT "2.2f";
			EDIT_FORMAT "5f";
			MIN_VALUE 20.00;
			MAX_VALUE 23.00;
			}
	 CONSTANT_UNIT "mA";
	 }


VARIABLE sensor_error
	 {
	 LABEL "Sens.Err"
				 "|049S.fehler";
	 HELP "Sensor breakdown indication, upscale 20.00 .. 23.00 mA , downscale 3.50 .. 4.00 "
				"|049Fhlerbruchangabe, upscale 20,00 .. 23,00 mA, downscale 3,50 .. 4,00 mA";

	 CLASS ANALOG_OUTPUT;
	 HANDLING READ&WRITE;
	 PRE_WRITE_ACTIONS
	 {
		pre_function
	 }
	 POST_WRITE_ACTIONS
	 {
		post_function
	 }

	 TYPE FLOAT
			{
				DISPLAY_FORMAT "2.2f";
				EDIT_FORMAT "5f";
				MIN_VALUE1 3.50;	MAX_VALUE1 4.00; MIN_VALUE2 20.00; MAX_VALUE2 23.00;
			}
	 CONSTANT_UNIT "mA";
	 }


COMMAND read_temperatur_sensor_alarm
	 {
	 NUMBER 134;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 under_min_sensor_value,over_max_sensor_value,sensor_error
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }


COMMAND write_temperatur_sensor_alarm
	 {
	 NUMBER 135;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 under_min_sensor_value,over_max_sensor_value,sensor_error
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 under_min_sensor_value,over_max_sensor_value,sensor_error
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			2,	DATA_ENTRY_ERROR,	[invalid_selection];
			5,	MISC_ERROR,			[too_few_data_bytes_recieved];
			}
	 }



VARIABLE min_value_messuret
 {
		LABEL
		SELECT (sensor_type)
		{
			CASE 1:
					"Min V";
			CASE 2:
					"Min V";
			CASE 3:
					"Min V";
			CASE 4:
					"Min V";
			CASE 5:
					"Min R";
			CASE 6:
					"Min R";
			CASE 7:
					"Min R";
			CASE 8:
					"Min R";
			CASE 9:
					"Min R";
			CASE 10:
					"Min R";
			CASE 11:
					"Min R";
			CASE 12:
					"Min R";
			DEFAULT:
					"Min T";
		}

	 HELP "Datalogger function, stores min value measured. Value can be cleared (Reset min/max)"
				"|049Dataloggerfunktion. Gemessener Mindestwert wird gespeichert. Wert kann zurckgesetzt werden. (min/max normieren)";

	 CLASS CORRECTION & DYNAMIC;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
		{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 1:
					".2f";
				CASE 2:
					".1f";
				CASE 3:
					".2f";
				CASE 4:
					".1f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".2f";
				CASE 14:
					".2f";
				CASE 15:
					".2f";
				CASE 28:
					".2f";
				CASE 29:
					".2f";
				DEFAULT:
					".1f";
			}

		}
 }


VARIABLE max_value_messuret
 {

		LABEL
		SELECT (sensor_type)
		{
			CASE 1:
					"Max V";
			CASE 2:
					"Max V";
			CASE 3:
					"Max V";
			CASE 4:
					"Max V";
			CASE 5:
					"Max R";
			CASE 6:
					"Max R";
			CASE 7:
					"Max R";
			CASE 8:
					"Max R";
			CASE 9:
					"Max R";
			CASE 10:
					"Max R";
			CASE 11:
					"Max R";
			CASE 12:
					"Max R";
			DEFAULT:
					"Max T";
		}

	 HELP "Datalogger function, stores max value measured. Value can be cleared (Reset min/max)"
				"|049Dataloggerfunktion. Gemessener max wert wird gespeichert. Wert kann zurckgesetzt werden.(min/max normieren)";
	 CLASS CORRECTION & DYNAMIC;
	 HANDLING READ&WRITE;

	 TYPE FLOAT
		{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 1:
					".2f";
				CASE 2:
					".1f";
				CASE 3:
					".2f";
				CASE 4:
					".1f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".2f";
				CASE 14:
					".2f";
				CASE 15:
					".2f";
				CASE 28:
					".2f";
				CASE 29:
					".2f";
				DEFAULT:
					".1f";
			}
		}
 }


COMMAND read_temperature_min_max_messuret
	 {
	 NUMBER 136;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 min_value_messuret,max_value_messuret
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }



COMMAND write_temperatur_reset_min_max_messuret
	 {
	 NUMBER 137;

	 OPERATION COMMAND;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status
				 }
			}

			RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			}
	 }


VARIABLE sensor_trim_value
{
	HELP "The offset added to the value measured"
			 "|049Offset ist zum Mewert addiert";
	CLASS INPUT;

	LABEL "Offs.";

	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 1:
					".2f";
				CASE 2:
					".1f";
				CASE 3:
					".2f";
				CASE 4:
					".1f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".2f";
				CASE 14:
					".2f";
				CASE 15:
					".2f";
				CASE 28:
					".2f";
				CASE 29:
					".2f";
				DEFAULT:
					".1f";
			}
		EDIT_FORMAT "7.f";
	}
}


VARIABLE sensor_trim_reset
	 {
	 CLASS INPUT;
	 LABEL "Local variable. Do not change"
				 "|049Lokale Variable. Darf nicht gendert werden";

	 HANDLING READ&WRITE;

	 TYPE UNSIGNED_INTEGER (1);
	 }


COMMAND write_sensor_trim
	 {
	 NUMBER 138;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 sensor_trim_value,sensor_trim_reset
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 sensor_trim_value,sensor_trim_reset
				 }
			}

			RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			}
	 }


COMMAND Read_sensor_trim
	 {
	 NUMBER 139;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 sensor_trim_value,sensor_trim_reset
				 }
			}

			RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			}
	 }


VARIABLE sensor_offset_value
{
	HELP "The offset added to the value measured" 
			 "|049Offset ist zum Mewert addiert";
	CLASS CORRECTION;
	LABEL "S.off.";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT
			SELECT (sensor_type)
			{
				CASE 1:
					".2f";
				CASE 2:
					".1f";
				CASE 3:
					".2f";
				CASE 4:
					".1f";
				CASE 5:
					".2f";
				CASE 7:
					".2f";
				CASE 9:
					".2f";
				CASE 11:
					".2f";
				CASE 13:
					".2f";
				CASE 14:
					".2f";
				CASE 15:
					".2f";
				CASE 28:
					".2f";
				CASE 29:
					".2f";
				DEFAULT:
					".1f";
			}
		EDIT_FORMAT "7.f";
	}
}


VARIABLE sensor_gain_value
{
	HELP "The gain is multiplied to the value measured, (if Unit is degF the zeropoint is 32 degF)"
			 "|049Gain ist mit dem Mewert multipliziert (wenn die Einheit degF ist,so ist der Nullpunkt 32 degF)";
	CLASS CORRECTION;
	LABEL "S.gain";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT "2.4f";
		EDIT_FORMAT "6f";
	}
}

COMMAND read_sensor_offset_gain
	 {
	 NUMBER 140;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 sensor_offset_value,sensor_gain_value
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }


COMMAND write_sensor_offset_gain
	 {
	 NUMBER 141;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 sensor_offset_value,sensor_gain_value
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 sensor_offset_value,sensor_gain_value
				 }
			}

	 RESPONSE_CODES
	 {
			0,	SUCCESS,			[no_command_specific_errors];
			2,	DATA_ENTRY_ERROR,	[invalid_selection];
			5,	MISC_ERROR,			[too_few_data_bytes_recieved];
			9,  DATA_ENTRY_ERROR, "Offset value too high""|049Offset-Wert ist zu hoch";
		 10,  DATA_ENTRY_ERROR, "Offset value too low""|049Offset-Wert ist zu niedrig";
		 11,  DATA_ENTRY_ERROR, "Gain value too high""|049Gain-Wert ist zu hoch";
		 12,  DATA_ENTRY_ERROR, "Gain value too low""|049Gain-Wert is zu niedrig";
		 13,  DATA_ENTRY_ERROR, "Offset and gain out of limits""|049Offset und Gain sind auerhalb der Grenzen";
			}
	 }



VARIABLE cjc_value
{
	HELP  "The CJC temperature" "|049CJC Temperatur";
	CLASS CORRECTION;
	LABEL "CJC t.";
	HANDLING READ&WRITE;
	TYPE FLOAT
	{
		DISPLAY_FORMAT "3.1f";
		EDIT_FORMAT "5f";
	}
	VALIDITY

	IF ((sensor_type > 15) && (sensor_type < 28) &&(wire_numbers == 1)) {TRUE;}
	ELSE {FALSE;}
}



VARIABLE cjc_units
{
	HELP [digital_units_help];
	CLASS CORRECTION;
	LABEL "Unit" "|049Einheit";
	HANDLING READ;
	TYPE ENUMERATED
	{
		ALL_UNITS
	}
}



COMMAND read_cjc_value
	 {
	 NUMBER 142;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 cjc_value,cjc_units
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,	[no_command_specific_errors];
			}
	 }


COMMAND write_cjc_value
	 {
	 NUMBER 143;

	 OPERATION WRITE;

	 TRANSACTION
			{
			REQUEST
				 {
				 cjc_value
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 cjc_value
				 }
			}

	 RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			2,	DATA_ENTRY_ERROR,	[invalid_selection];
			5,	MISC_ERROR,			[too_few_data_bytes_recieved];
			}
	 }

COMMAND read_dummy_unit
	 {
	 NUMBER 144;

	 OPERATION READ;

	 TRANSACTION
			{
			REQUEST
				 {
				 }

			REPLY
				 {
				 response_code,
				 device_status,
				 dummy_unit
				 }
			}

			RESPONSE_CODES
			{
			0,	SUCCESS,			[no_command_specific_errors];
			}
	 }


/*************************************************************************/
/*                   DEVICE SPECIFIC FUNCTION                            */
/*************************************************************************/

METHOD edit_type
	 {
	 LABEL "Sensor type"
				 "|049Sensortyp";

	 HELP "Select sensor type mv,Ohm,T/C,RTD "
				"|049Sensortyp mv, Ohm, T/C, RTD whlen";

	 DEFINITION
			{
			int x;
			int y;
			int z;
			int lokal_type;
			int lokal_var1;
			int lokal_var2;
			char status[STATUS_SIZE];
			float lokal_usl;
			float lokal_lsl;
			float	lokal_ms;
			int	  lokal_units;

/*********************** Select sensor type ******************************/

			IGNORE_ALL_DEVICE_STATUS();

			fassign(rtd_value,0);
			x=SELECT_FROM_LIST("Select sensortype" "|049Sensortyp whlen",
												 "RTD;T/C;mV;Ohm;Pot");
			switch (x)
				{
				case 0:
								y=SELECT_FROM_LIST("Select RTD type" "|049RTD-Typ whlen","Pt a=3850;Pt a=3902;Pt a=3916;NI a=1618;CU a=1428 ");
								if (y == 0) {lokal_type = y + 13;}
								else if ((y == 1) || (y == 2)){ lokal_type = y + 27;}
								else {lokal_type = y + 11;};
								fassign(rtd_value,100.0);
								GET_DEV_VAR_VALUE("Select RTD value " "|049RTD-Wert whlen",rtd_value);
								break;
				case 1:
								y=SELECT_FROM_LIST("Select T/C type " "|049T/C Typ whlen","B;E;J;K;L;N;R;S;T;U;W3;W5");
								lokal_type = y + 16;
								break;
				case 2:
								y=SELECT_FROM_LIST("Select mV Low/Hi" "|049 mV Low/Hi whlen" ,"-10 .. 70mV;-0.1 .. 1.1V");
								if (y==0)lokal_type = 1;
								else lokal_type = 2;
								break;
				case 3:
								y=SELECT_FROM_LIST("Select Ohm Low/Hi" "|049 Ohm Low/Hi whlen","0..390 Ohm;0..2200 Ohm");
								if (y==0) lokal_type = 5;
								else lokal_type = 6;
								break;
				case 4:
								y=SELECT_FROM_LIST("Select Pot Low/Hi" "|049 Pot Low/Hi whlen"," 0 .. 390 Ohm; 0 .. 2200 Ohm");
								if (y==0)lokal_type = 7;
								else lokal_type = 8;
								break;
				}

/*************************  init data if new type **************************/

				switch (lokal_type)
					{
					case 1:
									lokal_usl =70;lokal_lsl =-10;lokal_ms =2;lokal_units =36;
									lokal_var1 = 1; lokal_var2 = 2;
									break;
					case 2:
									lokal_usl =1100;lokal_lsl =-100;lokal_ms =20;lokal_units =36;
									lokal_var1 = 1; lokal_var2 = 0;
									break;
					case 5:
									lokal_usl =390;lokal_lsl =0;lokal_ms =5;lokal_units =37;
									lokal_var1 = 3; lokal_var2 = 1;
									break;
					case 6:
									lokal_usl =2200;lokal_lsl =0;lokal_ms =25;lokal_units =37;
									lokal_var1 = 3; lokal_var2 = 0;
									break;
					case 7:
									lokal_usl =390;lokal_lsl =0;lokal_ms =5;lokal_units =37;
									lokal_var1 = 5; lokal_var2 = 1;
									break;
					case 8:
									lokal_usl =2200;lokal_lsl =0;lokal_ms =25;lokal_units =37;
									lokal_var1 = 5; lokal_var2 = 0;
									break;
					case 13: case 28: case 29:
									if ((lokal_type == 13) || (lokal_type == 29)) {lokal_lsl =-200;} else {lokal_lsl = -150;};

									lokal_ms =10;lokal_units =32;
									if ((ivar_value(rtd_value) <= 500.0) && (lokal_type == 13)) lokal_usl =850;
									if ((ivar_value(rtd_value) <= 500.0) && (lokal_type == 28)) lokal_usl =720;
									if ((ivar_value(rtd_value) <= 500.0) && (lokal_type == 29)) lokal_usl =500;
									if ((ivar_value(rtd_value) <= 600.0) && (ivar_value(rtd_value)  > 500.0) && (lokal_type == 13))lokal_usl =750;
									if ((ivar_value(rtd_value) <= 700.0) && (ivar_value(rtd_value) > 600.0) && (lokal_type == 13))lokal_usl =580;
									if ((ivar_value(rtd_value) <= 700.0) && (ivar_value(rtd_value) > 500.0) && (lokal_type == 28))lokal_usl =580;
									if ((ivar_value(rtd_value) <= 800.0) && (ivar_value(rtd_value) > 700.0) && (lokal_type != 29))lokal_usl =480;
									if ((ivar_value(rtd_value) <= 800.0) && (ivar_value(rtd_value) > 500.0) && (lokal_type == 29))lokal_usl =480;
									if ((ivar_value(rtd_value) <= 900.0) && (ivar_value(rtd_value) > 800.0))lokal_usl =380;
									if  (ivar_value(rtd_value) > 900.0)lokal_usl =320;
									if (ivar_value(rtd_value) <= 100.0)
									{
										lokal_var1 = 4; lokal_var2 = 5;
									}
									else
									{
										lokal_var1 = 4; lokal_var2 = 4;
									};

									break;
					case 14:
									lokal_usl =250;lokal_lsl =-50;lokal_ms =10;lokal_units =32;
									if  (ivar_value(rtd_value) >= 700.0)lokal_usl =150;
									if (ivar_value(rtd_value) <= 100.0)
									{
										lokal_var1 = 4; lokal_var2 = 5;
									}
									else
									{
										lokal_var1 = 4; lokal_var2 = 4;
									};

									break;
					case 15:
									lokal_usl =200;lokal_lsl =-50;lokal_ms =10;lokal_units =32;
									if (ivar_value(rtd_value) <= 100.0)
										{
										 lokal_var1 = 4; lokal_var2 = 5;
										}
									else
										{
										 lokal_var1 = 4; lokal_var2 = 4;
										};

									break;
					case 16:
									lokal_usl =1820;lokal_lsl =100;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 17:
									lokal_usl =900;lokal_lsl =-270;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 18:
									lokal_usl =1200;lokal_lsl =-210;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 19:
									lokal_usl =1370;lokal_lsl =-250;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 20:
									lokal_usl =900;lokal_lsl =-200;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 21:
									lokal_usl =1300;lokal_lsl =-200;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 22: case 23:
									lokal_usl =1750;lokal_lsl =-50;lokal_ms =100;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 24:
									lokal_usl =400;lokal_lsl =-250;lokal_ms =40;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					case 25:
									lokal_usl =600;lokal_lsl =-200;lokal_ms =50;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					default:
									lokal_usl =2300;lokal_lsl =0;lokal_ms =100;lokal_units =32;
									lokal_var1 = 2; lokal_var2 = 7;
									break;
					}


				if ((lokal_units == 32) && (ivar_value(PV.DIGITAL_UNITS) == 33))
				{
						lokal_usl =(lokal_usl * 9 / 5) + 32;
						lokal_lsl =(lokal_lsl * 9 / 5) + 32;
						lokal_ms =(lokal_ms * 9 / 5);
						lokal_units =33;
				}

				fassign(PV.UPPER_SENSOR_LIMIT,lokal_usl);
				fassign(PV.LOWER_SENSOR_LIMIT,lokal_lsl);
				fassign(PV.MINIMUM_SPAN,lokal_ms);
				fassign(PV.UPPER_RANGE_VALUE,lokal_usl);
				fassign(PV.LOWER_RANGE_VALUE,lokal_lsl);
				assign_int(PV.DIGITAL_UNITS,lokal_units);
				assign_int(PV.RANGE_UNITS,lokal_units);
				assign_int(dummy_unit,lokal_units);

				fassign(min_value_messuret,lokal_usl);
				fassign(max_value_messuret,lokal_lsl);
				if ((lokal_type > 15) && (lokal_type < 28)) {assign_int(wire_numbers,0);}
				else {assign_int(wire_numbers,2);};
				fassign(cable_resistance,0);
				assign_int(sensor_type,lokal_type);
				assign_int(dummy_sensor_type,lokal_type);
				assign_int(sensor_type_var1,lokal_var1);
				assign_int(sensor_type_var2,lokal_var2);

				IGNORE_ALL_DEVICE_STATUS();
				/*IGNORE_ALL_RESPONSE_CODES();*/
				XMTR_IGNORE_ALL_DEVICE_STATUS();
				/*DELAY(1,"--PLEASE WAIT--" "|049--BITTE WARTEN--");*/

				send(131,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(131,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}
				send(44,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(44,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}

				send(34,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(34,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}
				send(35,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(35,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}

				send(133,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(133,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}

/*
				send(1,status);
				if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(1,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}
*/
				}
		}


/*************************************************************************/


METHOD reset_function
	 {
	 LABEL "Reset min/max"
				 "|049min/max  zurcksetzen";

	 HELP "Resets min/max value measured (datalogger function)"
				"|049Gemessener Min/Max Wert wird zurckgesetzt (Datalogger-Funktion)";

	 DEFINITION
			{
				char status[STATUS_SIZE];
				send(137,status);
				if(status[STATUS_RESPONSE_CODE])
				{
					display_response_status(137,status[STATUS_RESPONSE_CODE]);
					DELAY(2,"Aborting method due to last error");
					process_abort();
				}
				DELAY(2,"Min/max value has been cleared !" "|049min/max Wert ist zurckgesetzt! ");
			}
	 }


/*************************************************************************/


METHOD sensor_trim_reset_function
	 {
	 LABEL "Factory trim"
				 "|049Factory Trimm";

	 HELP "Factory trim resets the sensor trim offset/gain to factory setting"
				"|049Factory Trimm setzt Sensortrimm-Offset/gain zu Factory Setting zurck" ;

	 DEFINITION
			{
			 char status[STATUS_SIZE];


			 fassign(sensor_offset_value,0.0);
			 fassign(sensor_gain_value,1.0);
			 send(141,status);
			 if(status[STATUS_RESPONSE_CODE])
			 {
					display_response_status(141,status[STATUS_RESPONSE_CODE]);
					DELAY(2,"Aborting method due to last error");
					process_abort();
			 }
			 DELAY(4,"Sensor trim offset/gain has been reset to factory setting !"
							 "|049Sensortrimm Offset/gain ist auf Factory Setting zurckgesetzt.");
			 send(140,status);
			 if(status[STATUS_RESPONSE_CODE])
			 {
					display_response_status(140,status[STATUS_RESPONSE_CODE]);
					DELAY(2,"Aborting method due to last error");
					process_abort();
			 }
			 send(1,status);
			 if(status[STATUS_RESPONSE_CODE])
			 {
					display_response_status(1,status[STATUS_RESPONSE_CODE]);
					DELAY(2,"Aborting method due to last error");
					process_abort();
			 }
			 process_abort();
			}
	 }


/*************************************************************************/


METHOD one_point_sensor_trim
{
		HELP "Sensor offset Trim, Calibrate the sensor. The differens between the entered value and the measured value, may max differ 10% of USL-LSL"
				 "|049Sensor-Offset-Trimm. Kalibrierung des Sensors. Der Unterschied zwischen dem eingegebenen Wert und dem gemessenen Wert darf max. 10% von USL-LSL abweichen.";

		LABEL "One point trim""|049Ein-Punkt-Trimm.";
		DEFINITION
		{
			long ids[1];
			char status[STATUS_SIZE];
			float trim_point;
			float trim_point_mes;
			int rc;

			fassign(sensor_offset_value,0.0);
			fassign(sensor_gain_value,1.0);
			send(141,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(141,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			DELAY(4,"Sensor trim offset/gain has been reset to factory setting !"
							"|049Sensortrimm Offset/gain ist auf Factory Setting zurckgesetzt.");
			send(140,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(140,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			ACKNOWLEDGE("Apply input value" "|049Eingangswert anlegen");

			ids[0]=VARID(PV.DIGITAL_VALUE);
			acknowledge("Press OK when input is stable !   PV : %0""|049OK drcken wenn Eingang stabil ist  PV : %0" ,ids);

			send(1,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(1,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			trim_point = fvar_value(sensor_value);
			trim_point_mes = trim_point;
			fassign(sensor_gain_value,1.0);
			GET_LOCAL_VAR_VALUE("Enter applied value""|049Eingabe des angelegten Wertes", trim_point);

			if ( ( (trim_point) > (trim_point_mes+(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1))
			|| ((trim_point) < (trim_point_mes-(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1)) )
				{
					DELAY(4,"Aborting 1 point trim. Offset to large, max 10% of USL-LSL"
									"|049Abbruch des Ein-Punkt-Trimmes. Offset zu gro, max. 10& von USL-LSL.");
					process_abort();
				}
			else
			{
				fassign(sensor_offset_value, trim_point - trim_point_mes);
				send(141,status);
				if(status[STATUS_RESPONSE_CODE])
				{
					display_response_status(141,status[STATUS_RESPONSE_CODE]);
					DELAY(2,"Aborting method due to last error");
					process_abort();
				}
				DELAY(2,  " One point trim. ok""|049Ein-Punkt-Trimm. OK");
				process_abort();
			}
		}
}


/*************************************************************************/


METHOD two_point_sensor_trim
{
		HELP "Sensor offset/gain Trim, Calibrate the sensor. The differens between the entered value and the measured value may max differ 10% USL-LSL. Hi point should be minimum 0.5 % larger then low Point"
				 "|049Sensor-Offset/Gain-Trimm. Kalibrierung des Sensors. Der Unterschied zwischen dem eingegebenen Wert und dem gemessenen Wert darf max. 10% von USL-LSL abweichen. Hi-Punkt sollte minimum 0,5% grer als der niedrige Punkt sein.";

		LABEL "Two point trim""|049Zwei-Punkt-Trimm";
		DEFINITION
		{
			long ids[1];
			char status[STATUS_SIZE];
			float trim_point_hi_true;
			float trim_point_hi_mes;
			float trim_point_low_true;
			float trim_point_low_mes;
			float x;
			int rc;

			fassign(sensor_offset_value,0.0);
			fassign(sensor_gain_value,1.0);

			send(141,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(141,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			DELAY(4,"Sensor trim offset/gain has been reset to factory setting !"
							"|049Sensortrimm Offset/gain ist auf Factory Setting zurckgesetzt.");
			send(140,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(140,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			ACKNOWLEDGE("Apply low input value""|049Nidriegen Eingangswert anlegen");
			ids[0]=VARID(PV.DIGITAL_VALUE);
			acknowledge("Press OK when input is stable !   PV : %0"
																		"|049OK drcken wenn Eingang stabil ist  PV : %0" ,ids);
			send(1,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(1,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			trim_point_low_true = fvar_value(sensor_value);
			trim_point_low_mes = trim_point_low_true;
			GET_LOCAL_VAR_VALUE("Enter applied value""|049Eingabe des angelegten Wertes", trim_point_low_true);
			if ( ( (trim_point_low_true) > (trim_point_low_mes+(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1))
			|| ((trim_point_low_true) < (trim_point_low_mes-(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1)) )
				{
					DELAY(4,"Aborting two point trim. Offset too large, max 10% of USL-LSL"
									"|049Abbruch des Zwei-Punkt-Trimmes. Offset zu gro, max. 10% von USL-LSL");
					process_abort();
				}

			ACKNOWLEDGE("Apply hi input value""|049Hi-Eingangswert anlegen");
			ids[0]=VARID(PV.DIGITAL_VALUE);

			acknowledge("Press OK when input is stable !   PV : %0""|049OK drcken wenn Eingang stabil ist  PV : %0" ,ids);
			send(1,status);
			if(status[STATUS_RESPONSE_CODE])
			{
				display_response_status(1,status[STATUS_RESPONSE_CODE]);
				DELAY(2,"Aborting method due to last error");
				process_abort();
			}
			trim_point_hi_true = fvar_value(sensor_value);
			trim_point_hi_mes = trim_point_hi_true;
			GET_LOCAL_VAR_VALUE("Enter applied value""|049Eingabe des angelegten Wertes", trim_point_hi_true);

			if (((trim_point_hi_true) > (trim_point_hi_mes+(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1))
			|| ((trim_point_hi_true) < (trim_point_hi_mes-(fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit))* 0.1)))
				{
					DELAY(8,"Aborting two point trim. Offset too large, max 10% of USL-LSL"
									"|049Abbruch des Zwei-Punkt-Trimmes. Offset zu gro, max. 10% von USL-LSL");
					process_abort();
				}

			if ((trim_point_hi_mes - trim_point_low_mes) >= ((fvar_value(sensor_upper_limit) - fvar_value(sensor_lower_limit)) * 0.005))
				{
					fassign(sensor_gain_value, ((trim_point_hi_true - trim_point_low_true) / (trim_point_hi_mes - trim_point_low_mes) ));
					/*GET_DEV_VAR_VALUE("Enter gain ",sensor_gain_value);*/

					if (ivar_value(PV.DIGITAL_UNITS) == 33)
					{
						fassign(sensor_offset_value,(trim_point_low_true - (((trim_point_low_mes-32) * fvar_value(sensor_gain_value))+32)));
					}
					else
					{
						fassign(sensor_offset_value,(trim_point_low_true - (trim_point_low_mes * fvar_value(sensor_gain_value))));
					}
					send(141,status);
					if(status[STATUS_RESPONSE_CODE])
					{
						display_response_status(141,status[STATUS_RESPONSE_CODE]);
						DELAY(2,"Aborting method due to last error");
						process_abort();
					}
					DELAY(2, " Two point trim. ok""|049Zwei-Punkt Trimm. OK");
					process_abort();
				}
			else
				{
					DELAY(4,"Aborting two point trim. span to smal, hi point - low point has to be minimum 0.5% of USL-LSL"
									"|049Abbruch des Zwei-Punkt-Trimmes. Spanne zu klein. Hoher Punkt - niedriger Punkt mu mindestens 0,5% von USL-LSL unterschiedlich sein.");
					process_abort();
				}

		}
}


/*************************************************************************/

METHOD pre_function  /*No function */
	 {
	 LABEL "";

	 HELP "" ;

	 DEFINITION
			{
			 DELAY(0,"");
			}
	 }

/*************************************************************************/

METHOD post_function  /*No function*/
	 {
	 LABEL "";

	 HELP "" ;

	 DEFINITION
			{
			 DELAY(0,"");
			}
	 }

/*************************************************************************/

ARRAY OF COLLECTION dynamic_variables
{
	ELEMENTS
	{
		PRIMARY, pv, "";
	}
}

COLLECTION OF VARIABLE pv
{
	LABEL "";
	HELP [primary_variable_help];
	MEMBERS
	{
/* Sensor Correction Information */
		/* input_type, "Input Type", " Type = mv,Ohm,TC,RTD "*/
		DIGITAL_UNITS,				sensor_units;
		DIGITAL_VALUE,				sensor_value;
		SENSOR_UNITS,					sensor_units;
		cjc_unit,							sensor_units;
		UPPER_SENSOR_LIMIT,		sensor_upper_limit;
		LOWER_SENSOR_LIMIT,		sensor_lower_limit;
		MINIMUM_SPAN,					sensor_minimum_span;
		DAMPING_VALUE,				sensor_damping_value;
/* Analog Output Information */
		ANALOG_VALUE,					analog_value;
		ALARM_CODE,						analog_alarm_code;
/* Input Information */
		PERCENT_RANGE,				input_percent_range;
		RANGE_UNITS,					input_units;
		TRANSFER_FUNCTION,		input_transfer_function;
		UPPER_RANGE_VALUE,		input_upper_value;
		LOWER_RANGE_VALUE,		input_lower_value;
/* Device Information */
		SENSOR_SERIAL_NUMBER,	sensor_serial_number,	[sensor_serial_number], [sensor_serial_number_help];
	}
}


UNIT sensor_units_relation
{
	sensor_units:
	sensor_value,
	sensor_upper_limit,
	sensor_lower_limit,
	sensor_minimum_span,
	input_upper_value,
	input_lower_value,
	max_value_messuret,
	min_value_messuret,
	sensor_trim_value,
	sensor_offset_value,
	cjc_value
}

UNIT sensor_relation
{
	sensor_type:
	max_value_messuret,
	min_value_messuret,
	sensor_trim_value,
	sensor_offset_value
}



MENU root_menu
{
		LABEL "Main menu"
					"|049Hauptmen";
		ITEMS
		{
				device_setup_menu,                               /* menu */
				PV.DIGITAL_VALUE (DISPLAY_VALUE, READ_ONLY),     /* variable */
				PV.LOWER_RANGE_VALUE (DISPLAY_VALUE, READ_ONLY), /* variable */
				PV.UPPER_RANGE_VALUE (DISPLAY_VALUE, READ_ONLY),  /* variable */
				PV.ANALOG_VALUE (DISPLAY_VALUE, READ_ONLY),      /* variable */
				PV.PERCENT_RANGE (DISPLAY_VALUE, READ_ONLY),     /* variable */
				datalog_menu
		}
}

MENU device_setup_menu
{
	LABEL [device_setup];
		ITEMS
		{
				input_setup_menu,
				output_setup_menu,
				device_info_menu,
				diagnostic_menu
		}
}
MENU input_setup_menu
{
	LABEL "Input setup"
				"|049Eingang-Setup";
		ITEMS
		{
				signal_conditioning_menu,
				edit_type,
				PV.LOWER_RANGE_VALUE(DISPLAY_VALUE),       /* variable */
				PV.UPPER_RANGE_VALUE(DISPLAY_VALUE),       /* variable */
				PV.DAMPING_VALUE(DISPLAY_VALUE),           /* variable */
				sensor_units(DISPLAY_VALUE),               /* variable */
				wire_numbers(DISPLAY_VALUE),               /* variable */
				cjc_value(DISPLAY_VALUE),									 /* variable */
				cable_resistance(DISPLAY_VALUE)            /* variable */
		 }
}

MENU output_setup_menu
{
	LABEL "Output setup"
				"|049Ausgang-Setup";
	ITEMS
	{
				under_min_sensor_value(DISPLAY_VALUE),      /* variable */
				over_max_sensor_value(DISPLAY_VALUE),       /* variable */
				sensor_error(DISPLAY_VALUE)           			/* variable */
	}
}

MENU signal_conditioning_menu
{
	LABEL [signal_condition];
	ITEMS
	{
	sensor_type(DISPLAY_VALUE,READ_ONLY),
	PV.UPPER_SENSOR_LIMIT (DISPLAY_VALUE, READ_ONLY),
	PV.LOWER_SENSOR_LIMIT	(DISPLAY_VALUE, READ_ONLY),
	PV.MINIMUM_SPAN (DISPLAY_VALUE, READ_ONLY),
	rtd_value(DISPLAY_VALUE, READ_ONLY)
	}
}

MENU device_info_menu
{
		LABEL [device_information];
		ITEMS
		{
				private_label_distributor (DISPLAY_VALUE),
				device_type (DISPLAY_VALUE),
				device_id (DISPLAY_VALUE),
				tag (DISPLAY_VALUE),
				date (DISPLAY_VALUE),
				write_protect (DISPLAY_VALUE),
				descriptor,
				message,
				final_assembly_number,
				device_revisions_menu,
				hart_output_menu
		}
}

MENU device_revisions_menu
{
		LABEL [revision_numbers];
		ITEMS
		{
				universal_revision (DISPLAY_VALUE),
				transmitter_revision (DISPLAY_VALUE),
				software_revision (DISPLAY_VALUE)
		}
}


MENU hart_output_menu
{
	LABEL [HART_output];
	ITEMS
	{
		polling_address (DISPLAY_VALUE),        /* Variable */
		request_preambles (DISPLAY_VALUE)       /* Variable */
	}
}

MENU datalog_menu
{
	LABEL "DataLogger";
	ITEMS
	{
		max_value_messuret (DISPLAY_VALUE, READ_ONLY),
		min_value_messuret (DISPLAY_VALUE, READ_ONLY),
		reset_function
	}
}

MENU diagnostic_menu
{
	LABEL [diagnostics_and_service];
	ITEMS
	{
		loop_test,
		dac_trim,
		sensor_trim_menu
	}
}

MENU sensor_trim_menu
{
	LABEL "Sensor Trim";
	ITEMS
	{
		sensor_trim_reset_function,
		one_point_sensor_trim,
		two_point_sensor_trim,
		PV.DIGITAL_VALUE (DISPLAY_VALUE, READ_ONLY),
		sensor_offset_value(DISPLAY_VALUE),
		sensor_gain_value(DISPLAY_VALUE)
	}
}


MENU upload_variables
{
	 LABEL "";

	ITEMS
	{
	dummy_unit
	dummy_sensor_type,
	sensor_type_var1,
	sensor_type_var2,
	sensor_lower_limit,
	sensor_upper_limit,
	sensor_minimum_span,
	PV.DIGITAL_UNITS,

	PV.LOWER_RANGE_VALUE,
	PV.UPPER_RANGE_VALUE,
	PV.RANGE_UNITS,

	PV.DAMPING_VALUE,

	wire_numbers,
	rtd_value,
	cable_resistance,
	sensor_type,
	under_min_sensor_value,
	over_max_sensor_value,
	sensor_error,

	cjc_value,

	sensor_offset_value,
	sensor_gain_value,

	tag,
	descriptor,
	date,
	message,
	final_assembly_number,
	polling_address

	}
}

MENU hot_key
   {
	 LABEL "Quick Access Key" "|049Blitztaste";

	 ITEMS
      {
			input_setup_menu
			}
	 }

